@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -32,6 +30,9 @@
 #ifndef __MONITOR_HEADER__
 #define __MONITOR_HEADER__
 
+#include <sys/cdefs.h>
+
+__BEGIN_DECLS
 extern void monstartup (char *lowpc, char *highpc);
 
 extern void monitor (char *lowpc, char *highpc, char *buf, int bufsiz, int cntsiz);
@@ -43,5 +44,6 @@ extern void monoutput (const char *filename);
 extern void moninit (void);
 
 extern void monreset (void);
+__END_DECLS
 
 #endif	/* __MONITOR_HEADER__ */
